/*
 *  linux/drivers/fusion/isense.c
 *      Little linux driver / shim that interfaces with the linux mpi(core)
 *      driver to provide english readable strings in SCSI Error Report
 *      logging output.  This modules implement SCSI-3 Opcode lookup and
 *      a sorted table of SCSI-3 ASC/ASCQ strings.
 *
 *  Copyright (c) 1991-2000 Steven J. Ralston
 *  Written By: Steven J. Ralston
 *  (yes I wrote some of the orig. code back in 1991!)
 *  (mailto:Steve.Ralston@lsil.com)
 *
 *  $Id: isense.c,v 1.12 2000/09/27 10:16:59 sralston Exp $
 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    NO WARRANTY
    THE PROGRAM IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OR
    CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT
    LIMITATION, ANY WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT,
    MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Each Recipient is
    solely responsible for determining the appropriateness of using and
    distributing the Program and assumes all risks associated with its
    exercise of rights under this Agreement, including but not limited to
    the risks and costs of program errors, damage to or loss of data,
    programs or equipment, and unavailability or interruption of operations.

    DISCLAIMER OF LIABILITY
    NEITHER RECIPIENT NOR ANY CONTRIBUTORS SHALL HAVE ANY LIABILITY FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING WITHOUT LIMITATION LOST PROFITS), HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
    TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
    USE OR DISTRIBUTION OF THE PROGRAM OR THE EXERCISE OF ANY RIGHTS GRANTED
    HEREUNDER, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

// GRRRRRRRRRR...  FIXME!!!
#if defined(MODULE) && !defined(__GENKSYMS__)
#include <linux/module.h>
#endif

#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/version.h>
/* Jesh. */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0)
#include <asm/spinlock.h>
#endif

#define MoDuLeAuThOr "Steven J. Ralston"
#define CoPyRiGhT "Copyright (c) 2000 " MoDuLeAuThOr
#include "mpi.h"

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *  Private data...
 */

/*
 *  YIKES!  I don't usually #include C source files, but..
 *  The following #include pulls in our needed ASCQ_Table[]!
 */
#include "ascq_tbl.c"
#include "scsiops.c"

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
#define my_NAME     "SCSI-3 Opcodes & ASC/ASCQ Strings"
#define my_VERSION  LINUX_MPI_VERSION_COMMON

#ifdef MODULE
EXPORT_NO_SYMBOLS;
MODULE_AUTHOR(MoDuLeAuThOr);
MODULE_DESCRIPTION(my_NAME);
#endif

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
static int
dummy_func(void)
{
	return 1;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

#ifdef MODULE
int init_module(void)
#else
int __init isense_init(void)
#endif
{

    /*  Workaround for "mod->init out of bounds" bug in linux-2.4.0-test2-5  */
    (void) dummy_func();

    show_mpimod_ver(my_NAME, my_VERSION);

    /*
     *  Install our handler
     */
    if (mpi_register_ascqops_strings(&ASCQ_Table[0], ASCQ_TableSize, ScsiOpcodeString) != 1)
    {
        printk(KERN_ERR "isense: ERROR: Can't register with mpi(core) driver!\n");
        return -EBUSY;
    }
    printk(KERN_INFO "isense: Registered SCSI-3 Opcodes & ASC/ASCQ Strings\n");

    return 0;
}

#ifdef MODULE

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
void cleanup_module(void)
{
    mpi_deregister_ascqops_strings();
    printk(KERN_INFO "isense: Deregistered SCSI-3 Opcodes & ASC/ASCQ Strings\n");
}
 
#endif

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

