/*
 *  Copyright (c) 2000 LSI Logic Corporation.
 *
 *
 *           Name:  MPI_TARG.H
 *          Title:  MPI Target mode messages and structures
 *  Creation Date:  June 22, 2000
 *
 *    MPI Version:  01.00.02
 *
 *  Version History
 *  ---------------
 *   File Version:  %version: %
 *   Date Updated:  %date_modified: %
 *     Programmer:  %created_by: %
 *
 *
 *  Date      Version   Description
 *  --------  --------  ------------------------------------------------------
 *  05-08-00  00.10.01  Original release for 0.10 spec dated 4/26/2000.
 *  06-06-00  01.00.01  Update version number for 1.0 release.
 *  06-22-00  01.00.02  Added _MSG_TARGET_CMD_BUFFER_POST_REPLY structure.
 *                      Corrected DECSRIPTOR typo to DESCRIPTOR.
 *
 *  --------------------------------------------------------------------------
 */

#ifndef MPI_TARG_H
#define MPI_TARG_H


/******************************************************************************
*
*        S C S I    T a r g e t    M e s s a g e s
*
*******************************************************************************/

typedef struct _CMD_BUFFER_DESCRIPTOR
{
    U16                     HostIndex;
    U16                     Reserved;
    union
    {
        U32                 PhysicalAddress32;
        U64                 PhysicalAddress64;
    } u;
} CMD_BUFFER_DESCRIPTOR,
  CmdBufferDescriptor_t;


/****************************************************************************/
/* Target Command Buffer Post Request                                       */
/****************************************************************************/

typedef struct _MSG_TARGET_CMD_BUFFER_POST_REQUEST
{
    U8                      BufferPostFlags;
    U8                      BufferCount;
    U8                      ChainOffset;
    U8                      Function;
    U8                      BufferLength;
    U8                      Reserved;
    U8                      Reserved1;
    U8                      MsgFlags;
    U32                     MsgContext;
    CMD_BUFFER_DESCRIPTOR   Buffer[1];
} MSG_TARGET_CMD_BUFFER_POST_REQUEST, *PTR_MSG_TARGET_CMD_BUFFER_POST_REQUEST,
  TargetCmdBufferPostRequest_t, *pTargetCmdBufferPostRequest_t;

#define CMD_BUFFER_POST_FLAGS_PORT_MASK         (0x01)
#define CMD_BUFFER_POST_FLAGS_ADDR_MODE_MASK    (0x80)
#define CMD_BUFFER_POST_FLAGS_ADDR_MODE_32      (0)
#define CMD_BUFFER_POST_FLAGS_ADDR_MODE_64      (1)
#define CMD_BUFFER_POST_FLAGS_64_BIT_ADDR       (0x80)

#define CMD_BUFFER_POST_HOST_INDEX_MASK         (0x000003FF)


typedef struct _MSG_TARGET_CMD_BUFFER_POST_REPLY
{
    U8                      BufferPostFlags;
    U8                      BufferCount;
    U8                      MsgLength;
    U8                      Function;
    U8                      BufferLength;
    U8                      Reserved;
    U8                      Reserved1;
    U8                      MsgFlags;
    U32                     MsgContext;
    U16                     Reserved2;
    U16                     IOCStatus;
    U32                     IOCLogInfo;
} MSG_TARGET_CMD_BUFFER_POST_REPLY, *PTR_MSG_TARGET_CMD_BUFFER_POST_REPLY,
  TargetCmdBufferPostReply_t, *pTargetCmdBufferPostReply_t;


typedef struct _MSG_PRIORITY_CMD_RECEIVED_REPLY
{
    U16                     Reserved;
    U8                      MsgLength;
    U8                      Function;
    U16                     Reserved1;
    U8                      Reserved2;
    U8                      MsgFlags;
    U32                     MsgContext;
    U8                      PriorityReason;
    U8                      Reserved3;
    U16                     IOCStatus;
    U32                     IOCLogInfo;
    U32                     ReplyWord;
} MSG_PRIORITY_CMD_RECEIVED_REPLY, *PTR_MSG_PRIORITY_CMD_RECEIVED_REPLY,
  PriorityCommandReceivedReply_t, *pPriorityCommandReceivedReply_t;

#define PRIORITY_REASON_NO_DISCONNECT           (0x00)
#define PRIORITY_REASON_SCSI_TASK_MANAGEMENT    (0x01)
#define PRIORITY_REASON_UNKNOWN                 (0xFF)


typedef struct _MSG_TARGET_CMD_BUFFER_POST_ERROR_REPLY
{
    U16                     Reserved;
    U8                      MsgLength;
    U8                      Function;
    U16                     Reserved1;
    U8                      Reserved2;
    U8                      MsgFlags;
    U32                     MsgContext;
    U16                     Reserved3;
    U16                     IOCStatus;
    U32                     IOCLogInfo;
    U32                     ReplyWord;
} MSG_TARGET_CMD_BUFFER_POST_ERROR_REPLY,
  *PTR_MSG_TARGET_CMD_BUFFER_POST_ERROR_REPLY,
  TargetCmdBufferPostErrorReply_t, *pTargetCmdBufferPostErrorReply_t;


/****************************************************************************/
/* Target Assist Request                                                    */
/****************************************************************************/

typedef struct _MSG_TARGET_ASSIST_REQUEST
{
    U8                      StatusCode;
    U8                      TargetAssistFlags;
    U8                      ChainOffset;
    U8                      Function;
    U16                     QueueTag;
    U8                      Reserved;
    U8                      MsgFlags;
    U32                     MsgContext;
    U32                     ReplyWord;
    U8                      LUN[8];
    U32                     RelativeOffset;
    U32                     DataLength;
    SGE_IO_UNION            SGL[1];
} MSG_TARGET_ASSIST_REQUEST, *PTR_MSG_TARGET_ASSIST_REQUEST,
  TargetAssistRequest_t, *pTargetAssistRequest_t;

#define TARGET_ASSIST_FLAGS_DATA_DIRECTION  (0x01)
#define TARGET_ASSIST_FLAGS_AUTO_STATUS     (0x02)
#define TARGET_ASSIST_FLAGS_HIGH_PRIORITY   (0x04)


typedef struct _MSG_TARGET_ERROR_REPLY
{
    U16                     Reserved;
    U8                      MsgLength;
    U8                      Function;
    U16                     Reserved1;
    U8                      Reserved2;
    U8                      MsgFlags;
    U32                     MsgContext;
    U16                     Reserved3;
    U16                     IOCStatus;
    U32                     IOCLogInfo;
    U32                     ReplyWord;
    U32                     TransferCount;
} MSG_TARGET_ERROR_REPLY, *PTR_MSG_TARGET_ERROR_REPLY,
  TargetErrorReply_t, *pTargetErrorReply_t;


/****************************************************************************/
/* Target Status Send Request                                               */
/****************************************************************************/

typedef struct _MSG_TARGET_STATUS_SEND_REQUEST
{
    U8                      StatusCode;
    U8                      StatusFlags;
    U8                      ChainOffset;
    U8                      Function;
    U16                     QueueTag;
    U8                      Reserved;
    U8                      MsgFlags;
    U32                     MsgContext;
    U32                     ReplyWord;
    U8                      LUN[8];
    SGE_SIMPLE_UNION        StatusDataSGE;
} MSG_TARGET_STATUS_SEND_REQUEST, *PTR_MSG_TARGET_STATUS_SEND_REQUEST,
  TargetStatusSendRequest_t, *pTargetStatusSendRequest_t;

#define TARGET_STATUS_SEND_FLAGS_AUTO_GOOD_STATUS   (0x01)


/****************************************************************************/
/* Target Mode Abort Request                                                */
/****************************************************************************/

typedef struct _MSG_TARGET_MODE_ABORT_REQUEST
{
    U8                      AbortType;
    U8                      Reserved;
    U8                      ChainOffset;
    U8                      Function;
    U16                     Reserved1;
    U8                      Reserved2;
    U8                      MsgFlags;
    U32                     MsgContext;
    U32                     ReplyWord;
    U32                     MsgContextToAbort;
} MSG_TARGET_MODE_ABORT, *PTR_MSG_TARGET_MODE_ABORT,
  TargetModeAbort_t, *pTargetModeAbort_t;

#define TARGET_MODE_ABORT_TYPE_ALL_CMD_BUFFERS      (0x00)
#define TARGET_MODE_ABORT_TYPE_ALL_IO               (0x01)
#define TARGET_MODE_ABORT_TYPE_EXACT_IO             (0x02)
#define TARGET_MODE_ABORT_TYPE_EXACT_IO_REQUEST     (0x03)

/* Target Mode Abort Reply */

typedef struct _MSG_TARGET_MODE_ABORT_REPLY
{
    U16                     Reserved;
    U8                      MsgLength;
    U8                      Function;
    U16                     Reserved1;
    U8                      Reserved2;
    U8                      MsgFlags;
    U32                     MsgContext;
    U16                     Reserved3;
    U16                     IOCStatus;
    U32                     IOCLogInfo;
    U32                     AbortCount;
} MSG_TARGET_MODE_ABORT_REPLY, *PTR_MSG_TARGET_MODE_ABORT_REPLY,
  TargetModeAbortReply_t, *pTargetModeAbortReply_t;


/****************************************************************************/
/* Target Mode Context Reply                                                */
/****************************************************************************/

#define TARGET_MODE_REPLY_HOST_INDEX_MASK       (0x000003FF)
#define TARGET_MODE_REPLY_HOST_INDEX_SHIFT      (0)
#define TARGET_MODE_REPLY_IOC_INDEX_MASK        (0x001FF800)
#define TARGET_MODE_REPLY_IOC_INDEX_SHIFT       (11)
#define TARGET_MODE_REPLY_PORT_MASK             (0x00400000)
#define TARGET_MODE_REPLY_PORT_SHIFT            (22)
#define TARGET_MODE_REPLY_INITIATOR_INDEX_MASK  (0x1F800000)
#define TARGET_MODE_REPLY_INITIATOR_INDEX_SHIFT (23)

#define GET_HOST_INDEX(x)   (((x) & TARGET_MODE_REPLY_HOST_INDEX_MASK)         \
                                   >> TARGET_MODE_REPLY_HOST_INDEX_SHIFT)

#define SET_HOST_INDEX(t, hi)                                                  \
            ((t) = ((t) & ~TARGET_MODE_REPLY_HOST_INDEX_MASK) |                \
                             (((hi) << TARGET_MODE_REPLY_HOST_INDEX_SHIFT) &   \
                                           TARGET_MODE_REPLY_HOST_INDEX_MASK))

#define GET_IOC_INDEX(x)    (((x) & TARGET_MODE_REPLY_IOC_INDEX_MASK)          \
                                   >> TARGET_MODE_REPLY_IOC_INDEX_SHIFT)

#define SET_IOC_INDEX(t, ii)                                                   \
            ((t) = ((t) & ~TARGET_MODE_REPLY_IOC_INDEX_MASK) |                 \
                             (((ii) << TARGET_MODE_REPLY_IOC_INDEX_SHIFT) &    \
                                            TARGET_MODE_REPLY_IOC_INDEX_MASK))

#define GET_INITIATOR_INDEX(x) (((x) & TARGET_MODE_REPLY_INITIATOR_INDEX_MASK) \
                                   >> TARGET_MODE_REPLY_INITIATOR_INDEX_SHIFT)

#define SET_INITIATOR_INDEX(t, ii)                                             \
        ((t) = ((t) & ~TARGET_MODE_REPLY_INITIATOR_INDEX_MASK) |               \
                        (((ii) << TARGET_MODE_REPLY_INITIATOR_INDEX_SHIFT) &   \
                                      TARGET_MODE_REPLY_INITIATOR_INDEX_MASK))

#define GET_PORT(x) (((x) & TARGET_MODE_REPLY_PORT_MASK)                       \
                                               >> TARGET_MODE_REPLY_PORT_SHIFT)

#define SET_PORT(t, p)  ((t) = ((t) & ~TARGET_MODE_REPLY_PORT_MASK) |          \
                                    (((p) << TARGET_MODE_REPLY_PORT_SHIFT) &   \
                                                  TARGET_MODE_REPLY_PORT_MASK))


#endif

