/*
 *  linux/drivers/fusion/mpi.h
 *      High performance SCSI / Fibre Channel device driver.
 *      For use with PCI adapter(s):
 *          LSI SYM409xx (Fibre Channel)
 *      running LSI MPI (Message Passing Interface) firmware.
 *
 *  Credits:
 *      This driver would not exist if not for Alan Cox's development
 *      of the linux i2o driver.
 *      A huge debt of gratitude is owed to David S. Miller (DaveM)
 *      for fixing much of the stupid and broken stuff in the early
 *      driver while porting to sparc64 platform.  THANK YOU!
 *
 *  Copyright (c) 1999-2000 LSI Logic Corporation
 *  Originally By: Steven J. Ralston
 *  (mailto:Steve.Ralston@lsil.com)
 *
 *  $Id: mpi.h,v 1.53 2000/10/02 13:00:02 sralston Exp $
 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    NO WARRANTY
    THE PROGRAM IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OR
    CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT
    LIMITATION, ANY WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT,
    MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Each Recipient is
    solely responsible for determining the appropriateness of using and
    distributing the Program and assumes all risks associated with its
    exercise of rights under this Agreement, including but not limited to
    the risks and costs of program errors, damage to or loss of data,
    programs or equipment, and unavailability or interruption of operations.

    DISCLAIMER OF LIABILITY
    NEITHER RECIPIENT NOR ANY CONTRIBUTORS SHALL HAVE ANY LIABILITY FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING WITHOUT LIMITATION LOST PROFITS), HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
    TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
    USE OR DISTRIBUTION OF THE PROGRAM OR THE EXERCISE OF ANY RIGHTS GRANTED
    HEREUNDER, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef MPI_H_INCLUDED
#define MPI_H_INCLUDED
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

#include "linux_compat.h"

#include "lsi/mpi_type.h"
#include "lsi/mpi.h"
#include "lsi/mpi_ioc.h"
#include "lsi/mpi_init.h"
#include "lsi/mpi_cnfg.h"
#include "lsi/mpi_lan.h"
//#include "lsi/mpi_fc.h"
//#include "lsi/mpi_targ.h"

#include "symsense.h"

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

#ifndef MoDuLeAuThOr
#define MoDuLeAuThOr  "LSI Logic Corporation"
#endif

#ifndef CoPyRiGhT
#define CoPyRiGhT "Copyright (c) 1999-2000 " MoDuLeAuThOr
#endif

#define LINUX_MPI_VERSION_COMMON	"1.00.00-4"
#define LINUX_MPI_VERSION_EXP		"0.09.62-EXP"

#define show_mpimod_ver(s,ver)  \
    printk(KERN_INFO "%s %s\n", s, ver); \
    printk(KERN_INFO CoPyRiGhT "\n")

#define myMPI_MAX_ADAPTERS   16
#define myMPI_MAX_CALLBACKS  8

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

#ifdef MPI_DEBUG
#define STATIC
#define dprintk(x)  printk x
#else
#define STATIC      static
#define dprintk(x)
#endif

#ifdef MPI_DEBUG_HANDSHAKE
#define dhsprintk(x)  printk x
#else
#define dhsprintk(x)
#endif

#ifdef MPI_DEBUG_IRQ
#define dirqprintk(x)  printk x
#else
#define dirqprintk(x)
#endif

#ifdef MPI_DEBUG_SPINLOCK
#define dslprintk(x)  printk x
#else
#define dslprintk(x)
#endif

#ifdef MPI_DEBUG_SG
#define dsgprintk(x)  printk x
#else
#define dsgprintk(x)
#endif

#ifdef __KERNEL__

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

/*
 *  MPI adapter / port / bus / device info...
 */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

#define  myMPI_MAX_REQ_DEPTH     1023
#define  myMPI_REQ_DEPTH         1023
#define  myMPI_MIN_REQ_DEPTH      128

#define  myMPI_MAX_REPLY_DEPTH   1024
#define  myMPI_REPLY_DEPTH        128
#define  myMPI_MIN_REPLY_DEPTH      8
#define  myMPI_MAX_REPLIES_PER_ISR 10

#define  myMPI_MAX_FRAME_SIZE     128
#define  myMPI_REQ_SIZE           128
#define  myMPI_REPLY_SIZE         128

#define  MPI_SG_BUCKETS_PER_HUNK    1

#ifdef MODULE
#define  MPI_REQ_DEPTH_RANGE_STR    __MODULE_STRING(myMPI_MIN_REQ_DEPTH) "-" __MODULE_STRING(myMPI_MAX_REQ_DEPTH)
#define  MPI_REPLY_DEPTH_RANGE_STR  __MODULE_STRING(myMPI_MIN_REPLY_DEPTH) "-" __MODULE_STRING(myMPI_MAX_REPLY_DEPTH)
#define  MPI_REPLY_SIZE_RANGE_STR   __MODULE_STRING(myMPI_MIN_REPLY_SIZE) "-" __MODULE_STRING(myMPI_MAX_FRAME_SIZE)
#endif


typedef union _MPI_FRAME_TRACKER
{
	struct {
		struct _MPI_FRAME_HDR   *forw;
		struct _MPI_FRAME_HDR   *back;
		U32			 arg1;
		void			*argp1;
	} linkage;
	/*
	 * NOTE: On non-32-bit systems, where pointers are LARGE,
	 * using the linkage pointers destroys our sacred MsgContext
	 * field contents.  But we don't care anymore because these
	 * are now reset in MpiPutMsgFrame() just prior to sending
	 * a request off to the IOC.
	 */
	struct {
		U32 __hdr[2];
		/*
		 * The following _MUST_ match the location of the
		 * MsgContext field in the MPI message headers.
		 */
		union {
			U32                  MsgContext;
			struct {
				U16              req_idx;       /* Request index */
				U8               cb_idx;        /* callback function index */
				U8               rsvd;
			} fld;
		} msgctxu;
	} hwhdr;
} MPI_FRAME_TRACKER;

/*
 *  We might want to view/access a frame as:
 *    1) generic request header
 *    2) SCSIIORequest
 *    3) SCSIIOReply
 *    4) MPIDefaultReply
 *    5) frame tracker
 */
typedef struct _MPI_FRAME_HDR
{
    union {
        MPIHeader_t          hdr;
        SCSIIORequest_t      scsireq;
        SCSIIOReply_t        sreply;
        MPIDefaultReply_t    reply;
        MPI_FRAME_TRACKER    frame;
    } u;
} MPI_FRAME_HDR;

typedef struct _MPI_Q_TRACKER
{
    MPI_FRAME_HDR   *head;
    MPI_FRAME_HDR   *tail;
} MPI_Q_TRACKER;


typedef struct _MPI_SGL_HDR
{
    SGESimple32_t    sge[1];
} MPI_SGL_HDR;

typedef struct _MPI_SGL64_HDR
{
    SGESimple64_t    sge[1];
} MPI_SGL64_HDR;


typedef struct _Q_ITEM
{
    struct _Q_ITEM  *forw;
    struct _Q_ITEM  *back;
} Q_ITEM;

typedef struct _Q_TRACKER
{
    struct _Q_ITEM  *head;
    struct _Q_ITEM  *tail;
} Q_TRACKER;


/*
 *  Chip-specific stuff...
 */

typedef enum
{
    FC909 = 0x0909,
    FC929 = 0x0929,
    C1030 = 0x1030,
    FCUNK = 0x0BAD
} CHIP_TYPE;

/*
 *  System interface register set
 */

typedef struct _SYSIF_REGS
{
    U32     Doorbell;           /* 00     System<->IOC Doorbell reg  */
    U32     WriteSequence;      /* 04     Write Sequence register    */
    U32     Diagnostic;         /* 08     Diagnostic register        */
    U32     TestBase;           /* 0C     Test Base Address          */
    U32     Reserved1[8];       /* 10-2F  reserved for future use    */
    U32     IntStatus;          /* 30     Interrupt Status           */
    U32     IntMask;            /* 34     Interrupt Mask             */
    U32     Reserved2[2];       /* 38-3F  reserved for future use    */
    U32     RequestFifo;        /* 40     Request Post/Free FIFO     */
    U32     ReplyFifo;          /* 44     Reply   Post/Free FIFO     */
    U32     Reserved3[2];       /* 48-4F  reserved for future use    */
    U32     HostIndex;          /* 50     Host Index register        */
    U32     Reserved4[15];	/* 54-8F                             */
    U32     Fubar;              /* 90     For Fubar usage            */
    U32     Reserved5[27];	/* 94-FF                             */
} SYSIF_REGS;

/* little-endian! */

#define SYSIF_WRSEQ_MASK             0x0000000F  /*  4 bits */

#define SYSIF_DIAG_DIAGMEMORYMAP     0x00000001  /*  1 bit  */
#define SYSIF_DIAG_DISARM            0x00000002  /*  1 bit  */
#define SYSIF_DIAG_RESETADAPTER      0x00000004  /*  1 bit  */
#define SYSIF_DIAG_TTLINTERRUPT      0x00000008  /*  1 bit  */
#define SYSIF_DIAG_RESETHIST         0x00000020  /*  1 bit  */
#define SYSIF_DIAG_FLASHSIGERROR     0x00000040  /*  1 bit  */
#define SYSIF_DIAG_DIAGWRITEENABLE   0x00000080  /*  1 bit  */

#define SYSIF_TESTBASE_ADDR_MASK     0xFFFF0000  /* 16 bits */
#define SYSIF_TESTBASE_ADDR_SHIFT    16

#define SYSIF_INTSTAT_DOORBELLINT    0x00000001  /*  1 bit  */
#define SYSIF_INTSTAT_RESERVED1      0x00000006  /*  2 bits */
#define SYSIF_INTSTAT_REPLYINT       0x00000008  /*  1 bit  */
#define SYSIF_INTSTAT_RESERVED2      0xEFFFFFF0  /* 28 bits */
#define SYSIF_INTSTAT_IOPDBSTATUS    0x80000000  /*  1 bit  */

#define SYSIF_INTMASK_DOORBELLINT    0x00000001  /*  1 bit  */
#define SYSIF_INTMASK_RESERVED1      0x00000006  /*  2 bits */
#define SYSIF_INTMASK_REPLYINT       0x00000008  /*  1 bit  */
#define SYSIF_INTMASK_RESERVED2      0xFFFFFFF0  /* 28 bits */

#define SYSIF_HOSTINDEX_MASK         0x00003FFF  /* 14 bits */


typedef struct _MPI_ADAPTER
{
    struct _MPI_ADAPTER *forw;
    struct _MPI_ADAPTER *back;
    int                  id;			/* Unique adapter id {0,1,2,...} */
    int                  pci_irq;
    IOCFactsReply_t      facts0;
    IOCFactsReply_t      factsN;
    char                 name[32];		/* "iocN"             */
    U32                  mem_phys;		/* == f4020000 (mmap) */
    volatile SYSIF_REGS *chip;			/* == c8817000 (mmap) */
    CHIP_TYPE            chip_type;
    int                  mem_size;
    int                  alloc_total;
    U32                  last_state;
    int                  active;
    PortFactsReply_t     pfacts0;
    PortFactsReply_t     pfactsN;
    LANPage0_t           lan_cnfg_page0;
    LANPage1_t           lan_cnfg_page1;
    U8                  *reply_alloc;		/* Reply frames alloc ptr */
    dma_addr_t		 reply_alloc_dma;
    MPI_FRAME_HDR       *reply_frames;		/* Reply frames - rounded up! */
    dma_addr_t		 reply_frames_dma;
    int                  reply_depth;
    int                  reply_sz;
        /* We (host driver) get to manage our own RequestQueue! */
    U8                  *req_alloc;		/* Request frames alloc ptr */
    dma_addr_t		 req_alloc_dma;
    MPI_FRAME_HDR       *req_frames;		/* Request msg frames for PULL mode! */
    dma_addr_t		 req_frames_dma;
    int                  req_depth;
    int                  req_sz;
    spinlock_t		 FreeQlock;
    MPI_Q_TRACKER        FreeQ;

    /* Pool of SCSI sense buffers for commands coming from
     * the SCSI mid-layer.  We have one 256 byte sense buffer
     * for each REQ entry.
     */
    U8                  *sense_buf_pool;
    dma_addr_t           sense_buf_pool_dma;

    int                  hs_reply_idx;
    U32                  hs_req[128/sizeof(U32)];
    U16                  hs_reply[128/sizeof(U16)];

  /* OS-specific stuff */
    void                *pcidev;
    /*atomic_t             userCnt;*/
    U8                  *memmap;
  #ifdef CONFIG_MTRR
    int                  mtrr_reg;
  #endif
    struct Scsi_Host    *sh;
    struct proc_dir_entry *ioc_dentry;
} MPI_ADAPTER;


typedef struct _MPI_ADAPTER_TRACKER
{
    MPI_ADAPTER   *head;
    MPI_ADAPTER   *tail;
} MPI_ADAPTER_TRACKER;

/*
 *  New return value convention:
 *    1 = Ok to free associated request frame
 *    0 = not Ok ...
 */
typedef int (*MPI_CALLBACK)(MPI_ADAPTER *ioc, MPI_FRAME_HDR *req, MPI_FRAME_HDR *reply);


/*
 *  Fibre Channel (SCSI) target device...
 */
typedef struct _FC_TARGET
{
    struct _FC_TARGET   *forw;
    struct _FC_TARGET   *back;
    int                  bus_id;
    int                  target_id;
    int                  lun_exists[32];
    U8                   inquiry_data[36];
    U8                   last_sense[256];
} FC_TARGET;

typedef struct _FCDEV_TRACKER
{
    FC_TARGET           *head;
    FC_TARGET           *tail;
} FCDEV_TRACKER;


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

#define MPI_INDEX_2_MFPTR(ioc,idx) \
    (MPI_FRAME_HDR*)( (U8*)(ioc)->req_frames \
                      + (ioc)->req_sz * (idx) )

#define MFPTR_2_MPI_INDEX(ioc,mf) \
    (int)( ((U8*)mf - (U8*)(ioc)->req_frames) / (ioc)->req_sz )

#define Q_INIT(q,type)  (q)->head = (q)->tail = (type*)(q)
#define Q_IS_EMPTY(q)   ((Q_ITEM*)(q)->head == (Q_ITEM*)(q))

#define Q_ADD_TAIL(qt,i,type) \
{ \
    Q_TRACKER *_qt     = (Q_TRACKER*)(qt); \
    Q_ITEM    *oldTail = _qt->tail; \
\
    (i)->forw = (type*)_qt; \
    (i)->back = (type*)oldTail; \
    oldTail->forw = (Q_ITEM*)(i); \
    _qt->tail = (Q_ITEM*)(i); \
}

#define Q_ADD_HEAD(qt,i,type) \
{ \
    Q_TRACKER *_qt     = (Q_TRACKER*)(qt); \
    Q_ITEM    *oldHead = _qt->head; \
\
    (i)->forw = (type*)oldHead; \
    (i)->back = (type*)_qt; \
    oldHead->back = (Q_ITEM*)(i); \
    _qt->head = (Q_ITEM*)(i); \
}

#define Q_DEL_ITEM(i) \
{ \
    Q_ITEM  *_forw = (Q_ITEM*)(i)->forw; \
    Q_ITEM  *_back = (Q_ITEM*)(i)->back; \
\
    _back->forw = _forw; \
    _forw->back = _back; \
}


#define SWAB4(value) \
  (U32)(   (((value) & 0x000000ff) << 24) \
         | (((value) & 0x0000ff00) << 8)  \
         | (((value) & 0x00ff0000) >> 8)  \
         | (((value) & 0xff000000) >> 24) \
       )

#endif /* __KERNEL__ */


#ifndef MIN
#define MIN(a, b)   (((a) < (b)) ? (a) : (b))
#endif

#ifndef MAX
#define MAX(a, b)   (((a) > (b)) ? (a) : (b))
#endif


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

/*
 *  MPI Control IOCTLs and structures
 */
#define MPI_MAGIC_NUMBER  'm'
#define MPIRWPERF         _IOWR(MPI_MAGIC_NUMBER,0,struct mpi_raw_r_w)
#define MPIRWPERF_CHK     _IOR(MPI_MAGIC_NUMBER,13,struct mpi_raw_r_w)
#define MPIRWPERF_RESET   _IOR(MPI_MAGIC_NUMBER,14,struct mpi_raw_r_w)
#define MPIFWDOWNLOAD     _IOWR(MPI_MAGIC_NUMBER,15,struct mpi_fw_xfer)
#define MPISCSICMD        _IOWR(MPI_MAGIC_NUMBER,16,struct mpi_scsi_cmd)

/*
 *  Define something *vague* enough that caller doesn't
 *  really need to know anything about device parameters
 *  (blk_size, capacity, etc.)
 */
struct mpi_raw_r_w
{
    unsigned int     iocport;  /* IOC unit/port number */
    unsigned int     btl;      /* ?!?/Bus/Target/Lun */
    unsigned int     iters;    /* N iterations */
    unsigned short   nblks;    /* number of blocks per IO */
    unsigned short   qdepth;   /* max Q depth on this device */
    unsigned char    range;    /* 0-100% of FULL disk capacity, 0=use (nblks X iters) */
    unsigned char    skip;     /* % of disk to skip */
    unsigned char    rdwr;     /* 0-100%, 0=pure ReaDs, 100=pure WRites */
    unsigned char    seqran;   /* 0-100%, 0=pure SEQential, 100=pure RANdom */
    unsigned int     cache_sz; /* In Kb!  Optimize hits to N Kb cache size */
};

struct mpi_fw_xfer
{
/*    U8 flags;*/    /* Message flags - bit field */
    void *bufp;  /* Pointer to firmware buffer */
    unsigned int fwlen;
    int ioc;
};

struct mpi_scsi_cmd
{
    unsigned int     iocport;  /* IOC unit/port number */
    unsigned int     btl;      /* ?!?/Bus/Target/Lun */
    SCSIIORequest_t  scsi_req;
    SCSIIOReply_t    scsi_reply;
};

#ifdef __KERNEL__

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

/*
 *  Public entry points...
 */
extern int            mpi_register(MPI_CALLBACK cbfunc);
extern void           mpi_deregister(int cb_idx);
extern int            mpi_register_ascqops_strings(ASCQ_Table_t *, int, const char *[256]);
extern void           mpi_deregister_ascqops_strings(void);
extern MPI_FRAME_HDR *MpiGetMsgFrame(int handle, int iocid);
extern void           MpiFreeMsgFrame(int handle, int iocid, MPI_FRAME_HDR *mf);
extern void           MpiPutMsgFrame(int handle, int iocid, MPI_FRAME_HDR *mf);
extern int            SendHandShakeRequest(int handle, int iocid, int reqBytes, U32 *req);
extern int            mpi_verify_adapter(int iocid, MPI_ADAPTER **iocpp);
extern MPI_ADAPTER   *mpi_adapter_find_first(void);
extern MPI_ADAPTER   *mpi_adapter_find_next(MPI_ADAPTER *prev);
/*extern MPI_ADAPTER   *mpi_verify_device(int iocport, int btl);*/
extern void           mpi_print_ioc_facts(MPI_ADAPTER *ioc, char *buf, int *size, int len);

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

#endif /* __KERNEL__ */

#endif

