#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "mpi.h"

static char *ctlname = "/dev/mpictl";
static int step = 0;

/*
 *  Forward protos...
 */
static char *get_oldver(char *results_buf);
static char *what_search(const char *buf, int size_in, int *size_out);

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
int
main (int argc, char **argv)
{
	char userinp[128];
	char newver[64];
	char oldver[64];
	char what2[64];
	char *oldverp;
	struct mpi_fw_xfer fwdata;
	unsigned long ctlnum = MPIFWDOWNLOAD;
	char *fwbuf;
	int fwfd;
	int ctlfd;
	int size;
	struct stat fwstat;
	char *myname;
	char *fwfname = NULL;
	char *arg2 = NULL;
	char *whatstr;
	int whatsz;

	if ( (myname = strrchr(argv[0], '/')) == NULL ) {
		myname = argv[0];
	} else {
		myname++;
		argv[0] = myname;
	}
	if (argc > 1)
		fwfname = argv[1];
	if (argc > 2)
		arg2 = argv[2];
	else
		arg2 = "0";

	if (++step && (!fwfname || strlen(fwfname)<1) || !arg2 || !isdigit(arg2[0]))
	{
		fprintf(stderr, "Usage: %s fwfname.rom [0|1|2|...|7 for ioc# ]\n", myname);
		return step;
	}
	fprintf(stderr, "  step%d: Cmd line check: Ok\n", step);

	fwdata.ioc = atoi(arg2);
	if (fwdata.ioc > 8 || fwdata.ioc < 0) {
		fprintf(stderr, "  step%db: Cmd line Oops, resetting ioc from %d to 0\n", step, fwdata.ioc);
		fwdata.ioc = 0;
	}

        if (++step && (oldverp = get_oldver(oldver)) == NULL) {
		oldverp = "unknown";
	}
	fprintf(stderr, "  step%d: get_oldver() results: \"%s\"\n",
		    step, oldverp);

	/*
	 *  Open f/w image file
	 */
	if (++step && ((fwfd = open(fwfname, O_RDONLY)) < 0)) {
		fprintf(stderr, "ERROR: open(\"%s\") FAILED: %s\n", fwfname, strerror(errno));
		return step;
	}
	fprintf(stderr, "  step%d: open(\"%s\"): Ok, fwfd=%d\n", step, fwfname, fwfd);

	/*
	 *  Stat the f/w image file to get size
	 */
	if (++step && fstat(fwfd, &fwstat) < 0) {
		fprintf(stderr, "ERROR: fstat(%d) FAILED: %s\n",
			    fwfd, strerror(errno));
		return step;
	}
	size = (int)fwstat.st_size;
	fprintf(stderr, "  step%d: stat(\"%s\"): Ok, size=%d bytes\n",
		    step, fwfname, size);

	/* FIXME!
	 *  Check for current size restriction of 128 Kb!
	 */
	if (size > 128*1024) {
		fprintf(stderr, "ERROR: Bailing at step%d, size(=%d) > 128kB restriction! (FIXME!)\n",
			    step, size);
		return step;
	}

	/*
	 *  Malloc big enough buffer for entire f/w image
	 */
	if (++step && ((fwbuf = malloc(size)) == NULL)) {
		fprintf(stderr, "ERROR: Firmware buffer malloc %d bytes FAILED: %s\n",
			    size, strerror(errno));
		return step;
	}
	fprintf(stderr, "  step%d: malloc(%d): Ok\n", step, size);

	fwdata.bufp = fwbuf;
	fwdata.fwlen = size;

	/*
	 *  Read all bytes from f/w image file to malloc'd buffer
	 *  ToDo: *.rom files should have checksum value of 0!
	 *        Should verify that here.
	 */
	if (++step && (read(fwfd, fwbuf, size) < 0)) {
		fprintf(stderr, "ERROR: Firmware read from \"%s\" FAILED: %s\n",
			    fwfname, strerror(errno));
		return step;
	}
	fprintf(stderr, "  step%d: read(%d,,%d): Ok\n", step, fwfd, size);

	/*
	 *  Scan file for new version (magic what string identifier)
	 */
	newver[0] = what2[0] = '\0';
	if ((whatstr = what_search(fwbuf, size, &whatsz)) != NULL && whatsz) {
		strncpy(newver, whatstr, MIN(sizeof(newver)-1, whatsz));
		newver[MIN(sizeof(newver)-1, whatsz)] = '\0';
		if ((whatstr = what_search(whatstr, size-(whatstr-fwbuf+whatsz), &whatsz)) != NULL && whatsz) {
			strncpy(what2, whatstr, MIN(sizeof(what2)-1, whatsz));
			what2[MIN(sizeof(what2)-1, whatsz)] = '\0';
		}
	}
	fprintf(stderr, "         Current F/W Version = \"%s\"\n", oldverp);
	fprintf(stderr, "             New F/W Version = \"%s\"\n", newver);

	/*
	 *  Open /dev/mpictl for ioctl() call
	 */
	if (++step && ((ctlfd = open(ctlname, O_RDWR)) < 0)) {
		fprintf(stderr, "ERROR: open(\"%s\") FAILED: %s\n", ctlname, strerror(errno));
		return step;
	}
	fprintf(stderr, "  step%d: open(\"%s\"): Ok, ctlfd=%d\n", step, ctlname, ctlfd);

	++step;
	fprintf(stderr, "  pre-step%d: ioctl(%d,MPIFWDOWNLOAD,)\n", step, ctlfd);

	fprintf(stdout, "\n  Okay, this is like, a *really* DANGEROUS procedure.\n");
	fprintf(stdout, "\n  You are at irreversible step %d of %d in this process.\n",
		    step, step);
	fprintf(stdout, "  Are you absolutely sure you want to do this? (y/[n]) ");
	fgets(userinp, sizeof(userinp), stdin);

	if (userinp[0] == 'y') {
//fprintf(stderr, "DbG: fwdata.bufp  = %p\n", fwdata.bufp);
//fprintf(stderr, "DbG: fwdata.fwlen = %d\n", fwdata.fwlen);
//fprintf(stderr, "DbG: fwdata.ioc   = %04xh\n", fwdata.ioc);
		if (ioctl(ctlfd, ctlnum, (char *) &fwdata) < 0) {
			fprintf(stderr, "ERROR: Firmware transfer to ioc%d FAILED: %s\n",
				    fwdata.ioc, strerror(errno));
			return step;
		} else {
			fprintf(stdout, "\n  !!! MPI firmware transfer to ioc%d SUCCEEDED !!!\n", fwdata.ioc);
			fprintf(stdout, "  (%d of %d f/w update steps were successful)\n\n",
				    step, step);
			fprintf(stdout, "  CHANGE EFFECTIVE ONLY AFTER NEXT RESET / POWER CYCLE!\n\n");
		}
	} else {
		fprintf(stderr, "Bailed.\n");
	}
	
	return 0;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
static char *
get_oldver(char *results_buf)
{
	static char *procname = "/proc/mpi/info";
	static char *versearch = "FWVersion = ";
	char oldver_scratch[256];
	int procfd;
	unsigned int oldver_value = 0;
	char *str = NULL;

	/*
	 *  See if we can open "/proc/mpi/info" to obtain
	 *  current f/w version.
	 * FIXME!  Handle multiple MPI adapters!
	 *  Requires PROC_FS support obviously.
	 *  Maybe future MPI toolkit function?
	 */
	if ((procfd = open(procname, O_RDONLY)) >= 0) {
		char *found;
		/*
		 *  Dependency: Current format of /proc/mpi/info output:
		 *  |1|Attached MPI I/O Controllers:\n		// 30   30
                 *  |2|ioc0:\n					//  6   36
		 *  |3|  ProductID = 0x0621\n			// 21   57
		 *  |4|  PortNumber = 1 (of 1)\n		// 24   81
		 *  |5|  LanAddr = 0x(00:a0:b8:04:02:13)\n	// 34  115
		 *  |6|  FWVersion = 0xe60f  (Exp 0615)\n	<-- Line we want!
		 *  |7|  ...					// 33  148
		 */
		if (read(procfd, oldver_scratch, sizeof(oldver_scratch)) < 0) {
			fprintf(stderr, "         OldVer-read-fail-WARNING\n");
		} else {
			if ((found = strstr(oldver_scratch, versearch)) != NULL) {
				str = found + strlen(versearch);
				if (sscanf(str, "%x", &oldver_value) == 1) {
					;	// done!
				} else {
					fprintf(stderr, "         OldVer-sscanf-fail-WARNING\n");
					str = NULL;
				}
			} else {
				fprintf(stderr, "         OldVer-search-fail-WARNING\n");
			}
		}
		close(procfd);
	}

	if (oldver_value) {
		int mdbg = 0;

		if (oldver_value & 0x0080) {
			oldver_value &= ~0x0080;
			mdbg++;
		}

		if ((oldver_value & 0xE000) == 0xE000) {
			oldver_value &= 0x0FFFF;
			sprintf(results_buf, "SYMFC909-%d.%02d.%02d%s",
				    2000,
				    (oldver_value & 0x0F00) >> 8,
				    oldver_value & 0x00FF,
				    mdbg ? "  [MDBG]" : "" );
		} else {
			sprintf(results_buf, "SYMFC909-%d.%02d.%02d%s",
				    (oldver_value & 0xF000) >> 12,
				    (oldver_value & 0x0F00) >> 8,
				    oldver_value & 0x00FF,
				    mdbg ? "  [MDBG]" : "" );
		}
		str = results_buf;
	}

	return str;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
static char *
what_search(const char *buf, int size_in, int *size_out)
{
	register char c;
	register char *found = NULL;

	*size_out = 0;
	while (--size_in) {
		c = *buf++;
loop:
		if (size_in < 5)
			return NULL;
		if (c != '@')
			continue;
		if (--size_in && (c = *buf++) != '(')
			goto loop;
		if (--size_in && (c = *buf++) != '#')
			goto loop;
		if (--size_in && (c = *buf++) != ')')
			goto loop;
		found = (char*)buf;
		while (--size_in && (c = *buf++) && c != '"' && c != '>' && c != '\n')
			(*size_out)++;
//fprintf(stderr, "\nDbG: offset=%d, size_out=%d\n", found-fwbuf, *size_out);
		return found;
	}
	return NULL;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

