/*
 *  linux/drivers/fusion/mpiscsi.h
 *      High performance SCSI / Fibre Channel device driver.
 *      For use with PCI adapter(s):
 *          LSI SYM409xx (Fibre Channel)
 *      running LSI MPI (Message Passing Interface) firmware.
 *
 *  Credits:
 *      This driver would not exist if not for Alan Cox's development
 *      of the linux i2o driver.
 *      A huge debt of gratitude is owed to David S. Miller (DaveM)
 *      for fixing much of the stupid and broken stuff in the early
 *      driver while porting to sparc64 platform.  THANK YOU!
 *
 *  Copyright (c) 1999-2000 LSI Logic Corporation
 *  Originally By: Steven J. Ralston
 *  (mailto:Steve.Ralston@lsil.com)
 *
 *  $Id: mpiscsi.h,v 1.19 2000/09/27 10:17:04 sralston Exp $
 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    NO WARRANTY
    THE PROGRAM IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OR
    CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT
    LIMITATION, ANY WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT,
    MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Each Recipient is
    solely responsible for determining the appropriateness of using and
    distributing the Program and assumes all risks associated with its
    exercise of rights under this Agreement, including but not limited to
    the risks and costs of program errors, damage to or loss of data,
    programs or equipment, and unavailability or interruption of operations.

    DISCLAIMER OF LIABILITY
    NEITHER RECIPIENT NOR ANY CONTRIBUTORS SHALL HAVE ANY LIABILITY FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING WITHOUT LIMITATION LOST PROFITS), HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
    TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
    USE OR DISTRIBUTION OF THE PROGRAM OR THE EXERCISE OF ANY RIGHTS GRANTED
    HEREUNDER, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef MPISCSI_H_INCLUDED
#define MPISCSI_H_INCLUDED
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *  SCSI Public stuff...
 */

#ifdef __sparc__
#define MPI_SCSI_CAN_QUEUE       63
#define MPI_SCSI_CMD_PER_LUN     63
    /* FIXME!  Still investigating qd=64 hang on sparc64... */
#else
#define MPI_SCSI_CAN_QUEUE       64
#define MPI_SCSI_CMD_PER_LUN     64
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0)
#define PROC_SCSI_DECL
#else
#define PROC_SCSI_DECL  proc_name: "mpiscsi",
#endif

#ifdef MPI_SCSI_USE_NEW_EH

#define MPISCSI {                                    \
    next:                      NULL,                  \
    PROC_SCSI_DECL                                    \
    name:                      "MPI SCSI Initiator",  \
    detect:                    mpi_scsi_detect,       \
    release:                   mpi_scsi_release,      \
    info:                      mpi_scsi_info,         \
    command:                   NULL,                  \
    queuecommand:              mpi_scsi_queuecommand, \
    eh_strategy_handler:       NULL,                  \
    eh_abort_handler:          mpi_scsi_abort,        \
    eh_device_reset_handler:   mpi_scsi_dev_reset,    \
    eh_bus_reset_handler:      mpi_scsi_bus_reset,    \
    eh_host_reset_handler:     NULL,                  \
    bios_param:                mpi_scsi_bios_param,   \
    can_queue:                 MPI_SCSI_CAN_QUEUE,    \
    this_id:                   -1,                    \
    sg_tablesize:              25,                    \
    cmd_per_lun:               MPI_SCSI_CMD_PER_LUN,  \
    unchecked_isa_dma:         0,                     \
    use_clustering:            ENABLE_CLUSTERING,     \
    use_new_eh_code:           1                     \
}

#else

#define MPISCSI {                                    \
    next:                      NULL,                  \
    PROC_SCSI_DECL                                    \
    name:                      "MPI SCSI Initiator",  \
    detect:                    mpi_scsi_detect,       \
    release:                   mpi_scsi_release,      \
    info:                      mpi_scsi_info,         \
    command:                   NULL,                  \
    queuecommand:              mpi_scsi_queuecommand, \
    abort:                     mpi_scsi_old_abort,    \
    reset:                     mpi_scsi_old_reset,    \
    bios_param:                mpi_scsi_bios_param,   \
    can_queue:                 MPI_SCSI_CAN_QUEUE,    \
    this_id:                   -1,                    \
    sg_tablesize:              25,                    \
    cmd_per_lun:               MPI_SCSI_CMD_PER_LUN,  \
    unchecked_isa_dma:         0,                     \
    use_clustering:            ENABLE_CLUSTERING     \
}
#endif


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

/*  include/scsi/scsi.h may not be quite complete...  */
#ifndef RESERVE_10
#define RESERVE_10             0x56
#endif
#ifndef RELEASE_10
#define RELEASE_10             0x57
#endif
#ifndef PERSISTENT_RESERVE_IN
#define PERSISTENT_RESERVE_IN  0x5e
#endif
#ifndef PERSISTENT_RESERVE_OUT
#define PERSISTENT_RESERVE_OUT 0x5f
#endif


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

#endif

